% Best Approximation of a matrix by a frame multiplier
% function [uppsym TA] = ApprFramMult(T,D,Ds)
%
% usage:   uppsym=ApprFramMult(T,D)
%          [uppsym TA] = ApprFramMult(T,D,Ds)
%
% A matrix is best approximated (in the Hilbert-Schmidt sense) by a 
% frame multiplier. The elements of the frames are given in the synthesis
% matrices D and Ds columnswise. 
%  
% inputs : T .......... the matrix (m x n)
%          D .......... the elements of the analysis frame (columnwise) 
%                       (n x K)
%          Ds ......... the elements of the synthesis frame (columnwise) 
%                       (m x K)
%
% output: TA ......... the best apporximation of the matrix T with frame 
%                      multipliers using the frames in D and Ds
%         uppsym ..... symbols
%
% test:   T = eye(2,2)
%         D = [0 1/sqrt(2) -1/sqrt(2); 1 -1/sqrt(2) -1/sqrt(2)]
%         [coeff TA] = ApprFramMult(T,D);
%         See testapprfarmmult.m and testapprGabmultKap1.m
%
% date: 10/03/2005 - 03/04/2005
%
% notes : D and Ds are just the synthesis operators of the respective frames. 
%        To be able to define frame multipliers they have to have the same  
%        number of elements. 
%
% Author:  XXL .. Peter Balazs, contact: a8927259@unet.univie.ac.at
%
% Literature : [Xxl] P. Balazs; Irregular And Regular Gabor frame multipliers 
%                    with application to psychoacoustical masking 
%                    (Ph.D. thesis, in preparation 2005) 
%
% See also: GMappir,low2uppIr
%
% Copyright :  (c) NUHAG, Faculty of Math., University of Vienna, AUSTRIA
%              http://nuhag.mat.univie.ac.at/
%              (c) Acoustics Research Institute, Austrian Academy of
%              Science
%              http://www.kfs.oeaw.ac.at
function [uppsym,TA]=BestApprFramMult(T,D,Ds)

if nargin < 2
    error('At leatst two inputs are needed; T and D');
end;
[N M] = size(T);
[Nd Kd] = size(D);

if N ~= Nd
    error('The number of rows in D and T have to be the same.');
    % as T:C^m -> C^n and D=( g_1 g_2 ...) with g_i \in C^n 
end;

if nargin < 3
    Ds = D;
else
    [Ns Ks] = size(Ds);
    if N ~= Ns
        error('The number of rows in Ds and T have to be the same.');
    end;
    if Kd ~= Ks
        error('The frames must have the same number of elements.');
    end;
end;

lowsym = zeros(Kd,1); %lower symbol
for i=1:Kd
%    d = D(:,i);
%    ds = Ds(:,i);
%    lowsym(i) = conj(d'*(T*ds));
    lowsym(i) = conj(D(:,i)'*(T*Ds(:,i)));
end;
% the more elegant 
% lowsym = diag(D'*T*D) 
% is slower, O(k(n^2+n^2)))
% see [Xxl]

% Gram-matrix in Hilbert-Schmidt sense
if nargin < 3
    Gram = abs((D'*D)).^2;
else
    Gram = (D'*D).*((Ds'*Ds).');
end;

% upper symbol:
uppsym = pinv(Gram)*lowsym;

if nargout > 1
    % synthesis
    TA = zeros(N,M);
    for i = 1:Kd
    %    d = D(:,i); 
    %    ds = Ds(:,i);
    %    P = d*ds';
        P = D(:,i)*Ds(:,i)';
        TA = TA + uppsym(i)*P;
    end;
    % found no faster or more elgant way
end





